@@ -2,22 +2,23 @@
 # These two functions are simple dynamic programming algorithms for computing pairwise distances and are themselves used to validate
 # the RadixTree imeplementation (see test_radix_tree.R)
 
+# pwalign replaces Biostrings for alignment functions
+# We can now require pwalign instead of Biostrings since pwalign is now available on all R versions tested on CRAN
+# pwalign in 1.3.2 had a bug that causes test to fail, but 1.4.0 (R 4.5) fixes
+# https://github.com/Bioconductor/pwalign/issues/11
+# so only run if pwalign <= 1.2.0
+
 if(requireNamespace("seqtrie", quietly=TRUE) &&
    requireNamespace("stringi", quietly=TRUE) &&
    requireNamespace("stringdist", quietly=TRUE) &&
-   requireNamespace("Biostrings", quietly=TRUE) &&
    requireNamespace("dplyr", quietly=TRUE) && 
-   # pwalign is only required for Biostrings >= 2.72.0 in R 4.4+
-   ( (packageVersion("Biostrings") < "2.72.0") || requireNamespace("pwalign", quietly=TRUE) ) &&
-   # pwalign in devel (1.3.2) has a bug that causes test to fail
-   # https://github.com/Bioconductor/pwalign/issues/11
-   # so only run if Biostrings < 2.72.0 OR pwalign <= 1.2.0
-   ( (packageVersion("Biostrings") < "2.72.0") || (requireNamespace("pwalign", quietly=TRUE) && packageVersion("pwalign") <= "1.2.0") )
+   requireNamespace("pwalign", quietly=TRUE) &&
+   (packageVersion("pwalign") >= "1.4.0" || packageVersion("pwalign") <= "1.2.0")
 ) {
 library(seqtrie)
 library(stringi)
 library(stringdist)
-library(Biostrings)
+library(pwalign)
 library(dplyr)
 
 # Use 2 threads on github actions and CRAN, 4 threads locally
@@ -53,18 +54,6 @@ mutate_strings <- function(x, prob = 0.025, indel_prob = 0.025, charset = "abcde
   })
 }
 
-# Biostrings notes:
-# pairwiseAlignment is officially deprecated in Biostrings 2.75.1
-# but functionally deprecated as 2.72.0 as it still gives a warning which fails the test
-# use pwalign instead
-
-if(packageVersion("Biostrings") < "2.72.0") {
-  pairwiseAlignment_dispatch <- Biostrings::pairwiseAlignment
-} else {
-  pairwiseAlignment_dispatch <- pwalign::pairwiseAlignment
-}
-
-
 # subject (target) must be of length 1 or equal to pattern (query)
 # To get a distance matrix, iterate over target and perform a column bind
 # special_zero_case -- if both query and target are empty, Biostrings fails with an error
@@ -74,7 +63,7 @@ pairwiseAlignmentFix <- function(pattern, subject, ...) {
   if(all(special_zero_case)) {
     results
   } else {
-    results[!special_zero_case] <- pairwiseAlignment_dispatch(pattern=pattern[!special_zero_case], subject=subject[!special_zero_case], ...)
+    results[!special_zero_case] <- pwalign::pairwiseAlignment(pattern=pattern[!special_zero_case], subject=subject[!special_zero_case], ...)
     results
   }
 }
@@ -89,7 +78,7 @@ biostrings_matrix_global <- function(query, target, cost_matrix, gap_cost, gap_o
 
 biostrings_pairwise_global <- function(query, target, cost_matrix, gap_cost, gap_open_cost = 0) {
   substitutionMatrix <- -cost_matrix
-  -pairwiseAlignment_dispatch(pattern=query, subject=target, substitutionMatrix = substitutionMatrix,gapOpening=gap_open_cost, gapExtension=gap_cost, scoreOnly=TRUE, type="global")
+  -pairwiseAlignment(pattern=query, subject=target, substitutionMatrix = substitutionMatrix,gapOpening=gap_open_cost, gapExtension=gap_cost, scoreOnly=TRUE, type="global")
 }
 
 biostrings_matrix_anchored <- function(query, target, query_size, target_size, cost_matrix, gap_cost, gap_open_cost = 0) {
@@ -234,6 +223,7 @@ for(. in 1:NITER) {
     })
 
 
+
     print("Checking global search with affine gap for correctness")
     local({
       target <- c(random_strings(NSEQS, CHARSET),"") %>% unique
@@ -246,7 +236,7 @@ for(. in 1:NITER) {
       colnames(cost_matrix) <- rownames(cost_matrix) <- strsplit(CHARSET, "")[[1]]
       gap_cost <- sample(1:3, size = 1)
       gap_open_cost <- sample(1:3, size = 1)
-      results_seqtrie <- dist_matrix(query, target, mode = "levenshtein", cost_matrix = cost_matrix, gap_cost = gap_cost, gap_open_cos=gap_open_cost, nthreads=NTHREADS)
+      results_seqtrie <- dist_matrix(query, target, mode = "levenshtein", cost_matrix = cost_matrix, gap_cost = gap_cost, gap_open_cost=gap_open_cost, nthreads=NTHREADS)
       results_biostrings <- biostrings_matrix_global(query, target, cost_matrix = cost_matrix, gap_cost = gap_cost, gap_open_cost=gap_open_cost)
       stopifnot(all(results_seqtrie == results_biostrings))
 
